@@ -5,6 +5,7 @@ import (
 	"regexp"
 	"strings"
 
+	"github.com/grafana/alloy/internal/component/database_observability"
 	"github.com/prometheus/client_golang/prometheus"
 	"go.uber.org/atomic"
 )
@@ -22,6 +23,7 @@ type ConnectionInfoArguments struct {
 	DSN           string
 	Registry      *prometheus.Registry
 	EngineVersion string
+	CloudProvider *database_observability.CloudProvider
 }
 
 type ConnectionInfo struct {
@@ -29,6 +31,7 @@ type ConnectionInfo struct {
 	Registry      *prometheus.Registry
 	EngineVersion string
 	InfoMetric    *prometheus.GaugeVec
+	CloudProvider *database_observability.CloudProvider
 
 	running *atomic.Bool
 }
@@ -38,7 +41,7 @@ func NewConnectionInfo(args ConnectionInfoArguments) (*ConnectionInfo, error) {
 		Namespace: "database_observability",
 		Name:      "connection_info",
 		Help:      "Information about the connection",
-	}, []string{"provider_name", "provider_region", "db_instance_identifier", "engine", "engine_version"})
+	}, []string{"provider_name", "provider_region", "provider_account", "db_instance_identifier", "engine", "engine_version"})
 
 	args.Registry.MustRegister(infoMetric)
 
@@ -47,6 +50,7 @@ func NewConnectionInfo(args ConnectionInfoArguments) (*ConnectionInfo, error) {
 		Registry:      args.Registry,
 		EngineVersion: args.EngineVersion,
 		InfoMetric:    infoMetric,
+		CloudProvider: args.CloudProvider,
 		running:       &atomic.Bool{},
 	}, nil
 }
@@ -56,34 +60,45 @@ func (c *ConnectionInfo) Name() string {
 }
 
 func (c *ConnectionInfo) Start(ctx context.Context) error {
-	c.running.Store(true)
-
 	var (
 		providerName         = "unknown"
 		providerRegion       = "unknown"
+		providerAccount      = "unknown"
 		dbInstanceIdentifier = "unknown"
 		engine               = "postgres"
 		engineVersion        = "unknown"
 	)
 
-	parts, err := ParseURL(c.DSN)
-	if err != nil {
-		return err
-	}
-
-	if host, ok := parts["host"]; ok {
-		if strings.HasSuffix(host, "rds.amazonaws.com") {
+	if c.CloudProvider != nil {
+		if c.CloudProvider.AWS != nil {
 			providerName = "aws"
-			matches := rdsRegex.FindStringSubmatch(host)
-			if len(matches) > 3 {
-				dbInstanceIdentifier = matches[1]
-				providerRegion = matches[3]
+			providerAccount = c.CloudProvider.AWS.ARN.AccountID
+			providerRegion = c.CloudProvider.AWS.ARN.Region
+
+			// We only support RDS database for now. Resource types and ARN formats are documented at: https://docs.aws.amazon.com/service-authorization/latest/reference/list_amazonrds.html#amazonrds-resources-for-iam-policies
+			if resource := c.CloudProvider.AWS.ARN.Resource; strings.HasPrefix(resource, "db:") {
+				dbInstanceIdentifier = strings.TrimPrefix(resource, "db:")
 			}
-		} else if strings.HasSuffix(host, "postgres.database.azure.com") {
-			providerName = "azure"
-			matches := azureRegex.FindStringSubmatch(host)
-			if len(matches) > 1 {
-				dbInstanceIdentifier = matches[1]
+		}
+	} else {
+		parts, err := ParseURL(c.DSN)
+		if err != nil {
+			return err
+		}
+		if host, ok := parts["host"]; ok {
+			if strings.HasSuffix(host, "rds.amazonaws.com") {
+				providerName = "aws"
+				matches := rdsRegex.FindStringSubmatch(host)
+				if len(matches) > 3 {
+					dbInstanceIdentifier = matches[1]
+					providerRegion = matches[3]
+				}
+			} else if strings.HasSuffix(host, "postgres.database.azure.com") {
+				providerName = "azure"
+				matches := azureRegex.FindStringSubmatch(host)
+				if len(matches) > 1 {
+					dbInstanceIdentifier = matches[1]
+				}
 			}
 		}
 	}
@@ -93,7 +108,9 @@ func (c *ConnectionInfo) Start(ctx context.Context) error {
 		engineVersion = matches[1]
 	}
 
-	c.InfoMetric.WithLabelValues(providerName, providerRegion, dbInstanceIdentifier, engine, engineVersion).Set(1)
+	c.running.Store(true)
+
+	c.InfoMetric.WithLabelValues(providerName, providerRegion, providerAccount, dbInstanceIdentifier, engine, engineVersion).Set(1)
 	return nil
 }
 
