@@ -23,13 +23,13 @@ type LabelsConfig struct {
 }
 
 // validateLabelsConfig validates the Label stage configuration
-func validateLabelsConfig(c LabelsConfig) (map[string]string, error) {
+func validateLabelsConfig(c map[string]*string) (map[string]string, error) {
 	// We must not mutate the c.Values, create a copy with changes we need.
 	ret := map[string]string{}
-	if c.Values == nil {
+	if c == nil {
 		return nil, errors.New(ErrEmptyLabelStageConfig)
 	}
-	for labelName, labelSrc := range c.Values {
+	for labelName, labelSrc := range c {
 		// TODO: add support for different validation schemes.
 		//nolint:staticcheck
 		if !model.LabelName(labelName).IsValid() {
@@ -47,7 +47,7 @@ func validateLabelsConfig(c LabelsConfig) (map[string]string, error) {
 
 // newLabelStage creates a new label stage to set labels from extracted data
 func newLabelStage(logger log.Logger, configs LabelsConfig) (Stage, error) {
-	labelsConfig, err := validateLabelsConfig(configs)
+	labelsConfig, err := validateLabelsConfig(configs.Values)
 	if err != nil {
 		return nil, err
 	}
