@@ -21,38 +21,35 @@ const (
 	StageTypeDocker     = "docker"
 	StageTypeDrop       = "drop"
 	//TODO(thampiotr): Add support for eventlogmessage stage
-	StageTypeEventLogMessage    = "eventlogmessage"
-	StageTypeGeoIP              = "geoip"
-	StageTypeJSON               = "json"
-	StageTypeLabel              = "labels"
-	StageTypeLabelAllow         = "labelallow"
-	StageTypeLabelDrop          = "labeldrop"
-	StageTypeLimit              = "limit"
-	StageTypeLogfmt             = "logfmt"
-	StageTypeLuhn               = "luhn"
-	StageTypeMatch              = "match"
-	StageTypeMetric             = "metrics"
-	StageTypeMultiline          = "multiline"
-	StageTypeOutput             = "output"
-	StageTypePack               = "pack"
-	StageTypePattern            = "pattern"
-	StageTypePipeline           = "pipeline"
-	StageTypeRegex              = "regex"
-	StageTypeReplace            = "replace"
-	StageTypeSampling           = "sampling"
-	StageTypeStaticLabels       = "static_labels"
-	StageTypeStructuredMetadata = "structured_metadata"
-	StageTypeTemplate           = "template"
-	StageTypeTenant             = "tenant"
-	StageTypeTimestamp          = "timestamp"
-	StageTypeWindowsEvent       = "windowsevent"
+	StageTypeEventLogMessage        = "eventlogmessage"
+	StageTypeGeoIP                  = "geoip"
+	StageTypeJSON                   = "json"
+	StageTypeLabel                  = "labels"
+	StageTypeLabelAllow             = "labelallow"
+	StageTypeLabelDrop              = "labeldrop"
+	StageTypeLimit                  = "limit"
+	StageTypeLogfmt                 = "logfmt"
+	StageTypeLuhn                   = "luhn"
+	StageTypeMatch                  = "match"
+	StageTypeMetric                 = "metrics"
+	StageTypeMultiline              = "multiline"
+	StageTypeOutput                 = "output"
+	StageTypePack                   = "pack"
+	StageTypePattern                = "pattern"
+	StageTypePipeline               = "pipeline"
+	StageTypeRegex                  = "regex"
+	StageTypeReplace                = "replace"
+	StageTypeSampling               = "sampling"
+	StageTypeStaticLabels           = "static_labels"
+	StageTypeStructuredMetadata     = "structured_metadata"
+	StageTypeStructuredMetadataDrop = "structured_metadata_drop"
+	StageTypeTemplate               = "template"
+	StageTypeTenant                 = "tenant"
+	StageTypeTimestamp              = "timestamp"
+	StageTypeTruncate               = "truncate"
+	StageTypeWindowsEvent           = "windowsevent"
 )
 
-// Add stages that are not GA. Stages that are not specified here are considered GA.
-var stagesUnstable = map[string]featuregate.Stability{
-	StageTypeWindowsEvent: featuregate.StabilityExperimental,
-}
-
 // Processor takes an existing set of labels, timestamp and log entry and returns either a possibly mutated
 // timestamp and log entry
 type Processor interface {
@@ -119,14 +116,6 @@ func toStage(p Processor) Stage {
 	}
 }
 
-func checkFeatureStability(stageName string, minStability featuregate.Stability) error {
-	blockStability, exist := stagesUnstable[stageName]
-	if exist {
-		return featuregate.CheckAllowed(blockStability, minStability, fmt.Sprintf("stage %q", stageName))
-	}
-	return nil
-}
-
 // New creates a new stage for the given type and configuration.
 func New(logger log.Logger, jobName *string, cfg StageConfig, registerer prometheus.Registerer, minStability featuregate.Stability) (Stage, error) {
 	var (
@@ -174,6 +163,11 @@ func New(logger log.Logger, jobName *string, cfg StageConfig, registerer prometh
 		if err != nil {
 			return nil, err
 		}
+	case cfg.StructuredMetadataDropConfig != nil:
+		s, err = newStructuredMetadataDropStage(logger, *cfg.StructuredMetadataDropConfig)
+		if err != nil {
+			return nil, err
+		}
 	case cfg.RegexConfig != nil:
 		s, err = newRegexStage(logger, *cfg.RegexConfig)
 		if err != nil {
@@ -262,14 +256,15 @@ func New(logger log.Logger, jobName *string, cfg StageConfig, registerer prometh
 		if err != nil {
 			return nil, err
 		}
+	case cfg.TruncateConfig != nil:
+		s, err = newTruncateStage(logger, *cfg.TruncateConfig, registerer)
+		if err != nil {
+			return nil, err
+		}
 	default:
 		panic(fmt.Sprintf("unreachable; should have decoded into one of the StageConfig fields: %+v", cfg))
 	}
 
-	if err := checkFeatureStability(s.Name(), minStability); err != nil {
-		return nil, err
-	}
-
 	return s, nil
 }
 
