@@ -4,7 +4,6 @@ import (
 	"bufio"
 	"io"
 	"net/http"
-	"sort"
 	"strings"
 	"sync"
 	"time"
@@ -13,8 +12,9 @@ import (
 	"github.com/gorilla/mux"
 	"github.com/grafana/dskit/tenant"
 	"github.com/grafana/dskit/user"
+	lokipush "github.com/grafana/loki/pkg/push"
 	"github.com/grafana/loki/v3/pkg/loghttp/push"
-	"github.com/grafana/loki/v3/pkg/logproto"
+	"github.com/grafana/loki/v3/pkg/util/constants"
 	util_log "github.com/grafana/loki/v3/pkg/util/log"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/model"
@@ -33,7 +33,11 @@ type PushAPIServer struct {
 	logger       log.Logger
 	serverConfig *fnet.ServerConfig
 	server       *fnet.TargetServer
-	handler      loki.EntryHandler
+	handler      loki.LogsBatchReceiver
+	metrics      *metrics
+
+	once          sync.Once
+	forceShutdown chan struct{}
 
 	rwMutex            sync.RWMutex
 	labels             model.LabelSet
@@ -44,7 +48,7 @@ type PushAPIServer struct {
 
 func NewPushAPIServer(logger log.Logger,
 	serverConfig *fnet.ServerConfig,
-	handler loki.EntryHandler,
+	handler loki.LogsBatchReceiver,
 	registerer prometheus.Registerer,
 	maxSendMessageSize int64,
 ) (*PushAPIServer, error) {
@@ -58,6 +62,8 @@ func NewPushAPIServer(logger log.Logger,
 		logger:             logger,
 		serverConfig:       serverConfig,
 		handler:            handler,
+		metrics:            newMetircs(registerer),
+		forceShutdown:      make(chan struct{}),
 		maxSendMessageSize: maxSendMessageSize,
 	}
 
@@ -113,8 +119,32 @@ func (s *PushAPIServer) ServerConfig() fnet.ServerConfig {
 	return *s.serverConfig
 }
 
+func (s *PushAPIServer) HTTPListenAddress() string {
+	return s.server.HTTPListenAddr()
+}
+
+func (s *PushAPIServer) GRPCListenAddress() string {
+	return s.server.GRPCListenAddr()
+}
+
 func (s *PushAPIServer) Shutdown() {
 	level.Info(s.logger).Log("msg", "stopping push API server")
+	// StopAndShutdown tries to gracefully shutdown.
+	// It will stop idle and incoming connections
+	// and try to wait for all in-flight connections
+	// to finish. If configured timeout `ServerGracefulShutdownTimeout`
+	// expired this call will be unblocked.
+	s.server.StopAndShutdown()
+
+	// After we have tried a graceful shutdown we force all remaining in-flight
+	// requests to exit.
+	s.once.Do(func() { close(s.forceShutdown) })
+}
+
+// ForceShutdown will cancel all in-flight before starting server shutdown.
+func (s *PushAPIServer) ForceShutdown() {
+	level.Info(s.logger).Log("msg", "force shutdown of push API server")
+	s.once.Do(func() { close(s.forceShutdown) })
 	s.server.StopAndShutdown()
 }
 
@@ -175,6 +205,7 @@ func (s *PushAPIServer) handleLoki(w http.ResponseWriter, r *http.Request) {
 		nil, // usage tracker
 		nil,
 		"",
+		constants.Loki,
 	)
 	if err != nil {
 		level.Warn(s.logger).Log("msg", "failed to parse incoming push request", "err", err.Error())
@@ -187,14 +218,16 @@ func (s *PushAPIServer) handleLoki(w http.ResponseWriter, r *http.Request) {
 	relabelRules := s.getRelabelRules()
 	keepTimestamp := s.getKeepTimestamp()
 
-	var lastErr error
+	var (
+		entries []loki.Entry
+		lastErr error
+	)
 	for _, stream := range req.Streams {
 		ls, err := promql_parser.ParseMetric(stream.Labels)
 		if err != nil {
 			lastErr = err
 			continue
 		}
-		sort.Sort(ls)
 
 		lb := labels.NewBuilder(ls)
 
@@ -205,19 +238,18 @@ func (s *PushAPIServer) handleLoki(w http.ResponseWriter, r *http.Request) {
 
 		// Apply relabeling
 		processed, keep := relabel.Process(lb.Labels(), relabelRules...)
-		if !keep || len(processed) == 0 {
-			w.WriteHeader(http.StatusNoContent)
-			return
+		if !keep || processed.Len() == 0 {
+			continue
 		}
 
 		// Convert to model.LabelSet
 		filtered := model.LabelSet{}
-		for i := range processed {
-			if strings.HasPrefix(processed[i].Name, "__") {
-				continue
+		processed.Range(func(l labels.Label) {
+			if strings.HasPrefix(l.Name, "__") {
+				return
 			}
-			filtered[model.LabelName(processed[i].Name)] = model.LabelValue(processed[i].Value)
-		}
+			filtered[model.LabelName(l.Name)] = model.LabelValue(l.Value)
+		})
 
 		// Add tenant ID to the filtered labels if it is set
 		if tenantID != "" {
@@ -227,7 +259,7 @@ func (s *PushAPIServer) handleLoki(w http.ResponseWriter, r *http.Request) {
 		for _, entry := range stream.Entries {
 			e := loki.Entry{
 				Labels: filtered.Clone(),
-				Entry: logproto.Entry{
+				Entry: lokipush.Entry{
 					Line:               entry.Line,
 					StructuredMetadata: entry.StructuredMetadata,
 					Parsed:             entry.Parsed,
@@ -238,14 +270,30 @@ func (s *PushAPIServer) handleLoki(w http.ResponseWriter, r *http.Request) {
 			} else {
 				e.Timestamp = time.Now()
 			}
-			s.handler.Chan() <- e
+
+			entries = append(entries, e)
 		}
 	}
 
-	if lastErr != nil {
-		level.Warn(s.logger).Log("msg", "at least one entry in the push request failed to process", "err", lastErr.Error())
-		http.Error(w, lastErr.Error(), http.StatusBadRequest)
-		return
+	numEntries := len(entries)
+	if numEntries > 0 {
+		select {
+		case s.handler.Chan() <- entries:
+		case <-r.Context().Done():
+			w.WriteHeader(http.StatusServiceUnavailable)
+			return
+		case <-s.forceShutdown:
+			w.WriteHeader(http.StatusServiceUnavailable)
+			return
+		}
+
+		s.metrics.entriesWritten.Add(float64(numEntries))
+
+		if lastErr != nil {
+			level.Warn(s.logger).Log("msg", "at least one entry in the push request failed to process", "err", lastErr.Error())
+			http.Error(w, lastErr.Error(), http.StatusBadRequest)
+			return
+		}
 	}
 
 	w.WriteHeader(http.StatusNoContent)
@@ -254,10 +302,12 @@ func (s *PushAPIServer) handleLoki(w http.ResponseWriter, r *http.Request) {
 // NOTE: This code is copied from Promtail (https://github.com/grafana/loki/commit/47e2c5884f443667e64764f3fc3948f8f11abbb8) with changes kept to the minimum.
 // Only the HTTP handler functions are copied to allow for Alloy-specific server configuration and lifecycle management.
 func (s *PushAPIServer) handlePlaintext(w http.ResponseWriter, r *http.Request) {
-	entries := s.handler.Chan()
 	defer r.Body.Close()
 	body := bufio.NewReader(r.Body)
 	addLabels := s.getLabels()
+
+	var entries []loki.Entry
+
 	for {
 		line, err := body.ReadString('\n')
 		if err != nil && err != io.EOF {
@@ -272,18 +322,27 @@ func (s *PushAPIServer) handlePlaintext(w http.ResponseWriter, r *http.Request)
 			}
 			continue
 		}
-		entries <- loki.Entry{
-			Labels: addLabels,
-			Entry: logproto.Entry{
-				Timestamp: time.Now(),
-				Line:      line,
-			},
-		}
+
+		entries = append(entries, loki.Entry{Labels: addLabels, Entry: lokipush.Entry{Timestamp: time.Now(), Line: line}})
 		if err == io.EOF {
 			break
 		}
 	}
 
+	numEntries := len(entries)
+	if numEntries > 0 {
+		select {
+		case s.handler.Chan() <- entries:
+		case <-r.Context().Done():
+			w.WriteHeader(http.StatusServiceUnavailable)
+			return
+		case <-s.forceShutdown:
+			w.WriteHeader(http.StatusServiceUnavailable)
+			return
+		}
+		s.metrics.entriesWritten.Add(float64(numEntries))
+	}
+
 	w.WriteHeader(http.StatusNoContent)
 }
 
@@ -292,6 +351,6 @@ func (s *PushAPIServer) handlePlaintext(w http.ResponseWriter, r *http.Request)
 func (s *PushAPIServer) ready(w http.ResponseWriter, _ *http.Request) {
 	resp := "ready"
 	if _, err := w.Write([]byte(resp)); err != nil {
-		level.Error(s.logger).Log("msg", "failed to respond to ready endoint", "err", err)
+		level.Error(s.logger).Log("msg", "failed to respond to ready endpoint", "err", err)
 	}
 }
