@@ -31,6 +31,8 @@ type Interceptor struct {
 	// lastSeriesCount stores the number of series that were sent through the last interceptappender. It helps to estimate how
 	// much memory to allocate for the staleness trackers.
 	lastSeriesCount atomic.Int64
+
+	componentID string
 }
 
 var _ storage.Appendable = (*Interceptor)(nil)
@@ -91,6 +93,14 @@ func WithCTZeroSampleHook(f func(ref storage.SeriesRef, l labels.Labels, t, ct i
 	}
 }
 
+// WithComponentID returns an InterceptorOptions which is used to set the componentID of the Interceptor.
+// This is useful for debugging
+func WithComponentID(id string) InterceptorOption {
+	return func(i *Interceptor) {
+		i.componentID = id
+	}
+}
+
 // Appender satisfies the Appendable interface.
 func (f *Interceptor) Appender(ctx context.Context) storage.Appender {
 	app := &interceptappender{
@@ -104,6 +114,10 @@ func (f *Interceptor) Appender(ctx context.Context) storage.Appender {
 	return app
 }
 
+func (f *Interceptor) String() string {
+	return f.componentID + ".receiver"
+}
+
 type interceptappender struct {
 	interceptor       *Interceptor
 	child             storage.Appender
@@ -210,7 +224,7 @@ func (a *interceptappender) AppendHistogram(
 	if ref == 0 {
 		ref = storage.SeriesRef(a.ls.GetOrAddGlobalRefID(l))
 	}
-
+	// TODO histograms are not currently tracked for staleness causing them to be held forever
 	if a.interceptor.onAppendHistogram != nil {
 		return a.interceptor.onAppendHistogram(ref, l, t, h, fh, a.child)
 	}
