@@ -42,8 +42,8 @@ func (cg *ConfigGenerator) GenerateProbeConfig(m *promopv1.Probe) (cfg *config.S
 	if m.Spec.ProberSpec.Scheme != "" {
 		cfg.Scheme = m.Spec.ProberSpec.Scheme
 	}
-	if m.Spec.ProberSpec.ProxyURL != "" {
-		if u, err := url.Parse(m.Spec.ProberSpec.ProxyURL); err != nil {
+	if m.Spec.ProberSpec.ProxyURL != nil && *m.Spec.ProberSpec.ProxyURL != "" {
+		if u, err := url.Parse(*m.Spec.ProberSpec.ProxyURL); err != nil {
 			return nil, fmt.Errorf("parsing ProxyURL from probe: %w", err)
 		} else {
 			cfg.HTTPClientConfig.ProxyURL = commonConfig.URL{URL: u}
