@@ -10,6 +10,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/prometheus/common/model"
 	"github.com/prometheus/prometheus/config"
 	"github.com/prometheus/prometheus/model/relabel"
 	"github.com/prometheus/prometheus/scrape"
@@ -137,7 +138,7 @@ func (c *Config) ApplyDefaults(global GlobalConfig) error {
 		if sc.ScrapeProtocols == nil {
 			sc.ScrapeProtocols = c.global.Prometheus.ScrapeProtocols
 		}
-		if sc.MetricNameValidationScheme == "" {
+		if sc.MetricNameValidationScheme == model.UnsetValidation {
 			sc.MetricNameValidationScheme = c.global.Prometheus.MetricNameValidationScheme
 		}
 		if sc.MetricNameEscapingScheme == "" {
