@@ -600,99 +600,114 @@ func NewConfiguration() *Configuration {
 			},
 		},
 		unstableOperations: map[string]bool{
-			"v2.CreateOpenAPI":                       false,
-			"v2.DeleteOpenAPI":                       false,
-			"v2.GetOpenAPI":                          false,
-			"v2.ListAPIs":                            false,
-			"v2.UpdateOpenAPI":                       false,
-			"v2.CancelHistoricalJob":                 false,
-			"v2.ConvertJobResultToSignal":            false,
-			"v2.DeleteHistoricalJob":                 false,
-			"v2.GetFinding":                          false,
-			"v2.GetHistoricalJob":                    false,
-			"v2.GetRuleVersionHistory":               false,
-			"v2.GetSBOM":                             false,
-			"v2.ListAssetsSBOMs":                     false,
-			"v2.ListFindings":                        false,
-			"v2.ListHistoricalJobs":                  false,
-			"v2.ListVulnerabilities":                 false,
-			"v2.ListVulnerableAssets":                false,
-			"v2.MuteFindings":                        false,
-			"v2.RunHistoricalJob":                    false,
-			"v2.CreateDataset":                       false,
-			"v2.DeleteDataset":                       false,
-			"v2.GetAllDatasets":                      false,
-			"v2.GetDataset":                          false,
-			"v2.UpdateDataset":                       false,
-			"v2.CancelDataDeletionRequest":           false,
-			"v2.CreateDataDeletionRequest":           false,
-			"v2.GetDataDeletionRequests":             false,
-			"v2.CreateIncident":                      false,
-			"v2.CreateIncidentIntegration":           false,
-			"v2.CreateIncidentTodo":                  false,
-			"v2.CreateIncidentType":                  false,
-			"v2.DeleteIncident":                      false,
-			"v2.DeleteIncidentIntegration":           false,
-			"v2.DeleteIncidentTodo":                  false,
-			"v2.DeleteIncidentType":                  false,
-			"v2.GetIncident":                         false,
-			"v2.GetIncidentIntegration":              false,
-			"v2.GetIncidentTodo":                     false,
-			"v2.GetIncidentType":                     false,
-			"v2.ListIncidentAttachments":             false,
-			"v2.ListIncidentIntegrations":            false,
-			"v2.ListIncidents":                       false,
-			"v2.ListIncidentTodos":                   false,
-			"v2.ListIncidentTypes":                   false,
-			"v2.SearchIncidents":                     false,
-			"v2.UpdateIncident":                      false,
-			"v2.UpdateIncidentAttachments":           false,
-			"v2.UpdateIncidentIntegration":           false,
-			"v2.UpdateIncidentTodo":                  false,
-			"v2.UpdateIncidentType":                  false,
-			"v2.CreateAWSAccount":                    false,
-			"v2.CreateNewAWSExternalID":              false,
-			"v2.DeleteAWSAccount":                    false,
-			"v2.GetAWSAccount":                       false,
-			"v2.ListAWSAccounts":                     false,
-			"v2.ListAWSNamespaces":                   false,
-			"v2.UpdateAWSAccount":                    false,
-			"v2.ListAWSLogsServices":                 false,
-			"v2.CreateMonitorUserTemplate":           false,
-			"v2.DeleteMonitorUserTemplate":           false,
-			"v2.GetMonitorUserTemplate":              false,
-			"v2.ListMonitorUserTemplates":            false,
-			"v2.UpdateMonitorUserTemplate":           false,
-			"v2.ValidateExistingMonitorUserTemplate": false,
-			"v2.ValidateMonitorUserTemplate":         false,
-			"v2.CreatePipeline":                      false,
-			"v2.DeletePipeline":                      false,
-			"v2.GetPipeline":                         false,
-			"v2.ListPipelines":                       false,
-			"v2.UpdatePipeline":                      false,
-			"v2.ValidatePipeline":                    false,
-			"v2.CreateScorecardOutcomesBatch":        false,
-			"v2.CreateScorecardRule":                 false,
-			"v2.DeleteScorecardRule":                 false,
-			"v2.ListScorecardOutcomes":               false,
-			"v2.ListScorecardRules":                  false,
-			"v2.UpdateScorecardRule":                 false,
-			"v2.CreateIncidentService":               false,
-			"v2.DeleteIncidentService":               false,
-			"v2.GetIncidentService":                  false,
-			"v2.ListIncidentServices":                false,
-			"v2.UpdateIncidentService":               false,
-			"v2.CreateSLOReportJob":                  false,
-			"v2.GetSLOReport":                        false,
-			"v2.GetSLOReportJobStatus":               false,
-			"v2.AddMemberTeam":                       false,
-			"v2.ListMemberTeams":                     false,
-			"v2.RemoveMemberTeam":                    false,
-			"v2.CreateIncidentTeam":                  false,
-			"v2.DeleteIncidentTeam":                  false,
-			"v2.GetIncidentTeam":                     false,
-			"v2.ListIncidentTeams":                   false,
-			"v2.UpdateIncidentTeam":                  false,
+			"v2.CreateOpenAPI":                           false,
+			"v2.DeleteOpenAPI":                           false,
+			"v2.GetOpenAPI":                              false,
+			"v2.ListAPIs":                                false,
+			"v2.UpdateOpenAPI":                           false,
+			"v2.CancelHistoricalJob":                     false,
+			"v2.ConvertJobResultToSignal":                false,
+			"v2.DeleteHistoricalJob":                     false,
+			"v2.GetFinding":                              false,
+			"v2.GetHistoricalJob":                        false,
+			"v2.GetRuleVersionHistory":                   false,
+			"v2.GetSBOM":                                 false,
+			"v2.GetSecurityMonitoringHistsignal":         false,
+			"v2.GetSecurityMonitoringHistsignalsByJobId": false,
+			"v2.ListAssetsSBOMs":                         false,
+			"v2.ListFindings":                            false,
+			"v2.ListHistoricalJobs":                      false,
+			"v2.ListScannedAssetsMetadata":               false,
+			"v2.ListSecurityMonitoringHistsignals":       false,
+			"v2.ListVulnerabilities":                     false,
+			"v2.ListVulnerableAssets":                    false,
+			"v2.MuteFindings":                            false,
+			"v2.RunHistoricalJob":                        false,
+			"v2.SearchSecurityMonitoringHistsignals":     false,
+			"v2.CreateDataset":                           false,
+			"v2.DeleteDataset":                           false,
+			"v2.GetAllDatasets":                          false,
+			"v2.GetDataset":                              false,
+			"v2.UpdateDataset":                           false,
+			"v2.CancelDataDeletionRequest":               false,
+			"v2.CreateDataDeletionRequest":               false,
+			"v2.GetDataDeletionRequests":                 false,
+			"v2.CreateIncident":                          false,
+			"v2.CreateIncidentImpact":                    false,
+			"v2.CreateIncidentIntegration":               false,
+			"v2.CreateIncidentNotificationRule":          false,
+			"v2.CreateIncidentNotificationTemplate":      false,
+			"v2.CreateIncidentTodo":                      false,
+			"v2.CreateIncidentType":                      false,
+			"v2.DeleteIncident":                          false,
+			"v2.DeleteIncidentImpact":                    false,
+			"v2.DeleteIncidentIntegration":               false,
+			"v2.DeleteIncidentNotificationRule":          false,
+			"v2.DeleteIncidentNotificationTemplate":      false,
+			"v2.DeleteIncidentTodo":                      false,
+			"v2.DeleteIncidentType":                      false,
+			"v2.GetIncident":                             false,
+			"v2.GetIncidentIntegration":                  false,
+			"v2.GetIncidentNotificationRule":             false,
+			"v2.GetIncidentNotificationTemplate":         false,
+			"v2.GetIncidentTodo":                         false,
+			"v2.GetIncidentType":                         false,
+			"v2.ListIncidentAttachments":                 false,
+			"v2.ListIncidentImpacts":                     false,
+			"v2.ListIncidentIntegrations":                false,
+			"v2.ListIncidentNotificationRules":           false,
+			"v2.ListIncidentNotificationTemplates":       false,
+			"v2.ListIncidents":                           false,
+			"v2.ListIncidentTodos":                       false,
+			"v2.ListIncidentTypes":                       false,
+			"v2.SearchIncidents":                         false,
+			"v2.UpdateIncident":                          false,
+			"v2.UpdateIncidentAttachments":               false,
+			"v2.UpdateIncidentIntegration":               false,
+			"v2.UpdateIncidentNotificationRule":          false,
+			"v2.UpdateIncidentNotificationTemplate":      false,
+			"v2.UpdateIncidentTodo":                      false,
+			"v2.UpdateIncidentType":                      false,
+			"v2.CreateMonitorUserTemplate":               false,
+			"v2.DeleteMonitorUserTemplate":               false,
+			"v2.GetMonitorUserTemplate":                  false,
+			"v2.ListMonitorUserTemplates":                false,
+			"v2.UpdateMonitorUserTemplate":               false,
+			"v2.ValidateExistingMonitorUserTemplate":     false,
+			"v2.ValidateMonitorUserTemplate":             false,
+			"v2.ListRoleTemplates":                       false,
+			"v2.CreatePipeline":                          false,
+			"v2.DeletePipeline":                          false,
+			"v2.GetPipeline":                             false,
+			"v2.ListPipelines":                           false,
+			"v2.UpdatePipeline":                          false,
+			"v2.ValidatePipeline":                        false,
+			"v2.CreateScorecardOutcomesBatch":            false,
+			"v2.CreateScorecardRule":                     false,
+			"v2.DeleteScorecardRule":                     false,
+			"v2.ListScorecardOutcomes":                   false,
+			"v2.ListScorecardRules":                      false,
+			"v2.UpdateScorecardOutcomesAsync":            false,
+			"v2.UpdateScorecardRule":                     false,
+			"v2.CreateIncidentService":                   false,
+			"v2.DeleteIncidentService":                   false,
+			"v2.GetIncidentService":                      false,
+			"v2.ListIncidentServices":                    false,
+			"v2.UpdateIncidentService":                   false,
+			"v2.CreateSLOReportJob":                      false,
+			"v2.GetSLOReport":                            false,
+			"v2.GetSLOReportJobStatus":                   false,
+			"v2.GetSPARecommendations":                   false,
+			"v2.AddMemberTeam":                           false,
+			"v2.ListMemberTeams":                         false,
+			"v2.RemoveMemberTeam":                        false,
+			"v2.SyncTeams":                               false,
+			"v2.CreateIncidentTeam":                      false,
+			"v2.DeleteIncidentTeam":                      false,
+			"v2.GetIncidentTeam":                         false,
+			"v2.ListIncidentTeams":                       false,
+			"v2.UpdateIncidentTeam":                      false,
+			"v2.SearchFlakyTests":                        false,
 		},
 		RetryConfiguration: RetryConfiguration{
 			EnableRetry:       false,
