@@ -15,6 +15,7 @@ type Trigger struct {
 	CaseTriggerWrapper               *CaseTriggerWrapper
 	ChangeEventTriggerWrapper        *ChangeEventTriggerWrapper
 	DatabaseMonitoringTriggerWrapper *DatabaseMonitoringTriggerWrapper
+	DatastoreTriggerWrapper          *DatastoreTriggerWrapper
 	DashboardTriggerWrapper          *DashboardTriggerWrapper
 	GithubWebhookTriggerWrapper      *GithubWebhookTriggerWrapper
 	IncidentTriggerWrapper           *IncidentTriggerWrapper
@@ -56,6 +57,11 @@ func DatabaseMonitoringTriggerWrapperAsTrigger(v *DatabaseMonitoringTriggerWrapp
 	return Trigger{DatabaseMonitoringTriggerWrapper: v}
 }
 
+// DatastoreTriggerWrapperAsTrigger is a convenience function that returns DatastoreTriggerWrapper wrapped in Trigger.
+func DatastoreTriggerWrapperAsTrigger(v *DatastoreTriggerWrapper) Trigger {
+	return Trigger{DatastoreTriggerWrapper: v}
+}
+
 // DashboardTriggerWrapperAsTrigger is a convenience function that returns DashboardTriggerWrapper wrapped in Trigger.
 func DashboardTriggerWrapperAsTrigger(v *DashboardTriggerWrapper) Trigger {
 	return Trigger{DashboardTriggerWrapper: v}
@@ -200,6 +206,23 @@ func (obj *Trigger) UnmarshalJSON(data []byte) error {
 		obj.DatabaseMonitoringTriggerWrapper = nil
 	}
 
+	// try to unmarshal data into DatastoreTriggerWrapper
+	err = datadog.Unmarshal(data, &obj.DatastoreTriggerWrapper)
+	if err == nil {
+		if obj.DatastoreTriggerWrapper != nil && obj.DatastoreTriggerWrapper.UnparsedObject == nil {
+			jsonDatastoreTriggerWrapper, _ := datadog.Marshal(obj.DatastoreTriggerWrapper)
+			if string(jsonDatastoreTriggerWrapper) == "{}" { // empty struct
+				obj.DatastoreTriggerWrapper = nil
+			} else {
+				match++
+			}
+		} else {
+			obj.DatastoreTriggerWrapper = nil
+		}
+	} else {
+		obj.DatastoreTriggerWrapper = nil
+	}
+
 	// try to unmarshal data into DashboardTriggerWrapper
 	err = datadog.Unmarshal(data, &obj.DashboardTriggerWrapper)
 	if err == nil {
@@ -394,6 +417,7 @@ func (obj *Trigger) UnmarshalJSON(data []byte) error {
 		obj.CaseTriggerWrapper = nil
 		obj.ChangeEventTriggerWrapper = nil
 		obj.DatabaseMonitoringTriggerWrapper = nil
+		obj.DatastoreTriggerWrapper = nil
 		obj.DashboardTriggerWrapper = nil
 		obj.GithubWebhookTriggerWrapper = nil
 		obj.IncidentTriggerWrapper = nil
@@ -432,6 +456,10 @@ func (obj Trigger) MarshalJSON() ([]byte, error) {
 		return datadog.Marshal(&obj.DatabaseMonitoringTriggerWrapper)
 	}
 
+	if obj.DatastoreTriggerWrapper != nil {
+		return datadog.Marshal(&obj.DatastoreTriggerWrapper)
+	}
+
 	if obj.DashboardTriggerWrapper != nil {
 		return datadog.Marshal(&obj.DashboardTriggerWrapper)
 	}
@@ -504,6 +532,10 @@ func (obj *Trigger) GetActualInstance() interface{} {
 		return obj.DatabaseMonitoringTriggerWrapper
 	}
 
+	if obj.DatastoreTriggerWrapper != nil {
+		return obj.DatastoreTriggerWrapper
+	}
+
 	if obj.DashboardTriggerWrapper != nil {
 		return obj.DashboardTriggerWrapper
 	}
