@@ -12,6 +12,8 @@ import (
 
 // VulnerabilityAttributes The JSON:API attributes of the vulnerability.
 type VulnerabilityAttributes struct {
+	// Advisory associated with the vulnerability.
+	Advisory *VulnerabilityAdvisory `json:"advisory,omitempty"`
 	// Vulnerability advisory ID.
 	AdvisoryId *string `json:"advisory_id,omitempty"`
 	// Code vulnerability location.
@@ -91,6 +93,34 @@ func NewVulnerabilityAttributesWithDefaults() *VulnerabilityAttributes {
 	return &this
 }
 
+// GetAdvisory returns the Advisory field value if set, zero value otherwise.
+func (o *VulnerabilityAttributes) GetAdvisory() VulnerabilityAdvisory {
+	if o == nil || o.Advisory == nil {
+		var ret VulnerabilityAdvisory
+		return ret
+	}
+	return *o.Advisory
+}
+
+// GetAdvisoryOk returns a tuple with the Advisory field value if set, nil otherwise
+// and a boolean to check if the value has been set.
+func (o *VulnerabilityAttributes) GetAdvisoryOk() (*VulnerabilityAdvisory, bool) {
+	if o == nil || o.Advisory == nil {
+		return nil, false
+	}
+	return o.Advisory, true
+}
+
+// HasAdvisory returns a boolean if a field has been set.
+func (o *VulnerabilityAttributes) HasAdvisory() bool {
+	return o != nil && o.Advisory != nil
+}
+
+// SetAdvisory gets a reference to the given VulnerabilityAdvisory and assigns it to the Advisory field.
+func (o *VulnerabilityAttributes) SetAdvisory(v VulnerabilityAdvisory) {
+	o.Advisory = &v
+}
+
 // GetAdvisoryId returns the AdvisoryId field value if set, zero value otherwise.
 func (o *VulnerabilityAttributes) GetAdvisoryId() string {
 	if o == nil || o.AdvisoryId == nil {
@@ -610,6 +640,9 @@ func (o VulnerabilityAttributes) MarshalJSON() ([]byte, error) {
 	if o.UnparsedObject != nil {
 		return datadog.Marshal(o.UnparsedObject)
 	}
+	if o.Advisory != nil {
+		toSerialize["advisory"] = o.Advisory
+	}
 	if o.AdvisoryId != nil {
 		toSerialize["advisory_id"] = o.AdvisoryId
 	}
@@ -653,6 +686,7 @@ func (o VulnerabilityAttributes) MarshalJSON() ([]byte, error) {
 // UnmarshalJSON deserializes the given payload.
 func (o *VulnerabilityAttributes) UnmarshalJSON(bytes []byte) (err error) {
 	all := struct {
+		Advisory            *VulnerabilityAdvisory            `json:"advisory,omitempty"`
 		AdvisoryId          *string                           `json:"advisory_id,omitempty"`
 		CodeLocation        *CodeLocation                     `json:"code_location,omitempty"`
 		CveList             *[]string                         `json:"cve_list"`
@@ -725,12 +759,16 @@ func (o *VulnerabilityAttributes) UnmarshalJSON(bytes []byte) (err error) {
 	}
 	additionalProperties := make(map[string]interface{})
 	if err = datadog.Unmarshal(bytes, &additionalProperties); err == nil {
-		datadog.DeleteKeys(additionalProperties, &[]string{"advisory_id", "code_location", "cve_list", "cvss", "dependency_locations", "description", "ecosystem", "exposure_time", "first_detection", "fix_available", "language", "last_detection", "library", "origin", "remediations", "repo_digests", "risks", "status", "title", "tool", "type"})
+		datadog.DeleteKeys(additionalProperties, &[]string{"advisory", "advisory_id", "code_location", "cve_list", "cvss", "dependency_locations", "description", "ecosystem", "exposure_time", "first_detection", "fix_available", "language", "last_detection", "library", "origin", "remediations", "repo_digests", "risks", "status", "title", "tool", "type"})
 	} else {
 		return err
 	}
 
 	hasInvalidField := false
+	if all.Advisory != nil && all.Advisory.UnparsedObject != nil && o.UnparsedObject == nil {
+		hasInvalidField = true
+	}
+	o.Advisory = all.Advisory
 	o.AdvisoryId = all.AdvisoryId
 	if all.CodeLocation != nil && all.CodeLocation.UnparsedObject != nil && o.UnparsedObject == nil {
 		hasInvalidField = true
