@@ -137,10 +137,12 @@ func SpanPeer(span *Span) string {
 }
 
 func HostFromSchemeHost(span *Span) string {
-	if strings.Index(span.Statement, SchemeHostSeparator) > 0 {
-		schemeHost := strings.Split(span.Statement, SchemeHostSeparator)
-		if schemeHost[1] != "" {
-			return schemeHost[1]
+	if span.Type == EventTypeHTTPClient || span.Type == EventTypeHTTP {
+		if strings.Index(span.Statement, SchemeHostSeparator) > 0 {
+			schemeHost := strings.Split(span.Statement, SchemeHostSeparator)
+			if schemeHost[1] != "" {
+				return schemeHost[1]
+			}
 		}
 	}
 
