@@ -6,6 +6,8 @@ package otel
 import (
 	"context"
 
+	"go.opentelemetry.io/collector/exporter"
+	"go.opentelemetry.io/collector/pdata/ptrace"
 	sdkmetric "go.opentelemetry.io/otel/sdk/metric"
 	"go.opentelemetry.io/otel/sdk/metric/metricdata"
 
@@ -30,3 +32,18 @@ func (ie *instrumentedMetricsExporter) Export(ctx context.Context, md *metricdat
 	ie.internal.OTELMetricExport(totalMetrics)
 	return nil
 }
+
+// instrumentedTracesExporter wraps an otel traces exporter to account some internal metrics
+type instrumentedTracesExporter struct {
+	exporter.Traces
+	internal imetrics.Reporter
+}
+
+func (ie *instrumentedTracesExporter) ConsumeTraces(ctx context.Context, td ptrace.Traces) error {
+	if err := ie.Traces.ConsumeTraces(ctx, td); err != nil {
+		ie.internal.OTELTraceExportError(err)
+		return err
+	}
+	ie.internal.OTELTraceExport(td.SpanCount())
+	return nil
+}
