@@ -100,6 +100,8 @@ func converters[K any]() []ottl.Factory[K] {
 		NewTimeFactory[K](),
 		NewFormatTimeFactory[K](),
 		NewTrimFactory[K](),
+		NewTrimPrefixFactory[K](),
+		NewTrimSuffixFactory[K](),
 		NewToKeyValueStringFactory[K](),
 		NewToCamelCaseFactory[K](),
 		NewToLowerCaseFactory[K](),
@@ -125,5 +127,7 @@ func converters[K any]() []ottl.Factory[K] {
 		NewProfileIDFactory[K](),
 		NewParseIntFactory[K](),
 		NewKeysFactory[K](),
+		NewXXH3Factory[K](),
+		NewXXH128Factory[K](),
 	}
 }
