@@ -33,7 +33,11 @@ const (
 	MetadataFromDeployment = "deployment"
 	// MetadataFromStatefulSet is used to specify to extract metadata/labels/annotations from statefulset
 	MetadataFromStatefulSet = "statefulset"
-	PodIdentifierMaxLength  = 4
+	// MetadataFromDaemonSet  is used to specify to extract metadata/labels/annotations from daemonset
+	MetadataFromDaemonSet = "daemonset"
+	// MetadataFromJob  is used to specify to extract metadata/labels/annotations from job
+	MetadataFromJob        = "job"
+	PodIdentifierMaxLength = 4
 
 	ResourceSource   = "resource_attribute"
 	ConnectionSource = "connection"
@@ -97,6 +101,8 @@ type Client interface {
 	GetNode(string) (*Node, bool)
 	GetDeployment(string) (*Deployment, bool)
 	GetStatefulSet(string) (*StatefulSet, bool)
+	GetDaemonSet(string) (*DaemonSet, bool)
+	GetJob(string) (*Job, bool)
 	Start() error
 	Stop()
 }
@@ -120,6 +126,8 @@ type Pod struct {
 	NodeName       string
 	DeploymentUID  string
 	StatefulSetUID string
+	DaemonSetUID   string
+	JobUID         string
 	HostNetwork    bool
 
 	// Containers specifies all containers in this pod.
@@ -173,9 +181,9 @@ type Node struct {
 type deleteRequest struct {
 	// id is identifier (IP address or Pod UID) of pod to remove from pods map
 	id PodIdentifier
-	// name contains name of pod to remove from pods map
-	podName string
-	ts      time.Time
+	// contains uid of pod to remove from pods map
+	podUID string
+	ts     time.Time
 }
 
 // Filters is used to instruct the client on how to filter out k8s pods.
@@ -224,6 +232,7 @@ type LabelFilter struct {
 // from pods and added to the spans as tags.
 type ExtractionRules struct {
 	CronJobName               bool
+	CronJobUID                bool
 	DeploymentName            bool
 	DeploymentUID             bool
 	DaemonSetUID              bool
@@ -253,14 +262,16 @@ type ExtractionRules struct {
 	ServiceVersion            bool
 	ServiceInstanceID         bool
 
-	Annotations []FieldExtractionRule
-	Labels      []FieldExtractionRule
+	Annotations                  []FieldExtractionRule
+	Labels                       []FieldExtractionRule
+	DeploymentNameFromReplicaSet bool
 }
 
 // IncludesOwnerMetadata determines whether the ExtractionRules include metadata about Pod Owners
 func (rules *ExtractionRules) IncludesOwnerMetadata() bool {
 	rulesNeedingOwnerMetadata := []bool{
 		rules.CronJobName,
+		rules.CronJobUID,
 		rules.DeploymentName,
 		rules.DeploymentUID,
 		rules.DaemonSetUID,
@@ -300,6 +311,8 @@ type FieldExtractionRule struct {
 	//  - node
 	//  - deployment
 	//  - statefulset
+	//  - daemonset
+	//  - job
 	From string
 }
 
@@ -334,6 +347,18 @@ func (r *FieldExtractionRule) extractFromStatefulSetMetadata(metadata, tags map[
 	}
 }
 
+func (r *FieldExtractionRule) extractFromDaemonSetMetadata(metadata, tags map[string]string, formatter string) {
+	if r.From == MetadataFromDaemonSet {
+		r.extractFromMetadata(metadata, tags, formatter)
+	}
+}
+
+func (r *FieldExtractionRule) extractFromJobMetadata(metadata, tags map[string]string, formatter string) {
+	if r.From == MetadataFromJob {
+		r.extractFromMetadata(metadata, tags, formatter)
+	}
+}
+
 func (r *FieldExtractionRule) extractFromMetadata(metadata, tags map[string]string, formatter string) {
 	if r.KeyRegex != nil {
 		for k, v := range metadata {
@@ -415,6 +440,28 @@ type StatefulSet struct {
 	Attributes map[string]string
 }
 
+// DaemonSet represents a kubernetes daemonset.
+type DaemonSet struct {
+	Name       string
+	UID        string
+	Attributes map[string]string
+}
+
+// Job represents a kubernetes job.
+type Job struct {
+	Name       string
+	UID        string
+	Attributes map[string]string
+	CronJob    CronJob
+}
+
+// CronJob represents a kubernetes cronjob.
+type CronJob struct {
+	Name       string
+	UID        string
+	Attributes map[string]string
+}
+
 func OtelAnnotations() FieldExtractionRule {
 	return FieldExtractionRule{
 		Name:                 "$1",
