@@ -5,6 +5,7 @@ package sampling // import "github.com/open-telemetry/opentelemetry-collector-co
 
 import (
 	"context"
+	"fmt"
 	"regexp"
 
 	"github.com/golang/groupcache/lru"
@@ -12,6 +13,8 @@ import (
 	"go.opentelemetry.io/collector/pdata/pcommon"
 	"go.opentelemetry.io/collector/pdata/ptrace"
 	"go.uber.org/zap"
+
+	"github.com/open-telemetry/opentelemetry-collector-contrib/processor/tailsamplingprocessor/pkg/samplingpolicy"
 )
 
 const defaultCacheSize = 128
@@ -30,17 +33,20 @@ type regexStrSetting struct {
 	filterList   []*regexp.Regexp
 }
 
-var _ PolicyEvaluator = (*stringAttributeFilter)(nil)
+var _ samplingpolicy.Evaluator = (*stringAttributeFilter)(nil)
 
 // NewStringAttributeFilter creates a policy evaluator that samples all traces with
 // the given attribute in the given numeric range.
-func NewStringAttributeFilter(settings component.TelemetrySettings, key string, values []string, regexMatchEnabled bool, evictSize int, invertMatch bool) PolicyEvaluator {
+func NewStringAttributeFilter(settings component.TelemetrySettings, key string, values []string, regexMatchEnabled bool, evictSize int, invertMatch bool) (samplingpolicy.Evaluator, error) {
 	// initialize regex filter rules and LRU cache for matched results
 	if regexMatchEnabled {
 		if evictSize <= 0 {
 			evictSize = defaultCacheSize
 		}
-		filterList := addFilters(values)
+		filterList, err := addFilters(values)
+		if err != nil {
+			return nil, err
+		}
 		regexStrSetting := &regexStrSetting{
 			matchedAttrs: lru.New(evictSize),
 			filterList:   filterList,
@@ -66,7 +72,7 @@ func NewStringAttributeFilter(settings component.TelemetrySettings, key string,
 				return false
 			},
 			invertMatch: invertMatch,
-		}
+		}, nil
 	}
 
 	// initialize the exact value map
@@ -85,13 +91,13 @@ func NewStringAttributeFilter(settings component.TelemetrySettings, key string,
 			return matched
 		},
 		invertMatch: invertMatch,
-	}
+	}, nil
 }
 
 // Evaluate looks at the trace data and returns a corresponding SamplingDecision.
 // The SamplingDecision is made by comparing the attribute values with the matching values,
 // which might be static strings or regular expressions.
-func (saf *stringAttributeFilter) Evaluate(_ context.Context, _ pcommon.TraceID, trace *TraceData) (Decision, error) {
+func (saf *stringAttributeFilter) Evaluate(_ context.Context, _ pcommon.TraceID, trace *samplingpolicy.TraceData) (samplingpolicy.Decision, error) {
 	saf.logger.Debug("Evaluating spans in string-tag filter")
 	trace.Lock()
 	defer trace.Unlock()
@@ -149,11 +155,14 @@ func (saf *stringAttributeFilter) Evaluate(_ context.Context, _ pcommon.TraceID,
 
 // addFilters compiles all the given filters and stores them as regexes.
 // All regexes are automatically anchored to enforce full string matches.
-func addFilters(exprs []string) []*regexp.Regexp {
+func addFilters(exprs []string) ([]*regexp.Regexp, error) {
 	list := make([]*regexp.Regexp, 0, len(exprs))
 	for _, entry := range exprs {
-		rule := regexp.MustCompile(entry)
+		rule, err := regexp.Compile(entry)
+		if err != nil {
+			return nil, fmt.Errorf("invalid regex `%s`: %w", entry, err)
+		}
 		list = append(list, rule)
 	}
-	return list
+	return list, nil
 }
