@@ -23,6 +23,6 @@ type Config struct {
 var _ component.Config = (*Config)(nil)
 
 // Validate checks the receiver configuration is valid
-func (cfg *Config) Validate() error {
+func (*Config) Validate() error {
 	return nil
 }
